// external lib
import { ListenerRss, ListenerRSSInfos } from "listener-rss";

// local lib
import { ManageListener } from "../";

// Unit test
import path from "path";
import events from "events";
import nock from "nock";
import * as chai from "chai";
import sinon from "ts-sinon";
import sinonChai from "sinon-chai";

chai.use(sinonChai);

const expect = chai.expect;

const premakeInfo: ListenerRSSInfos[] = [
  {
    address:
      "https://www.youtube.com/feeds/videos.xml?channel_id=UCOuIgj0CYCXCvjWywjDbauw",
    timeloop: 10,
    customfields: {
      "media:group": ["media:group"],
    },
  },
  {
    address:
      "https://www.youtube.com/feeds/videos.xml?channel_id=UCh2YBKhYIy-_LtfCIn2Jycg",
    timeloop: 20,
    customfields: {
      "media:group": ["media:group"],
    },
  },
  {
    address:
      "https://www.youtube.com/feeds/videos.xml?channel_id=UCY7klexd1qEqxgqYK6W7BVQ",
    timeloop: 30,
    customfields: { "media:group": ["media:group"] },
  },
];

describe("test class GestionListener", function () {
  describe("test constructor", function () {
    it("should create a new manageListener object without params", function () {
      // when
      const ml = new ManageListener();

      // expect
      expect(ml.listenerArray).to.be.empty;
    });

    it("should create a new manageListener object with a param file", function () {
      // given
      const tmp = require("./RessourcesTest/FakeRessources/save.json");

      // when
      const ml = new ManageListener({
        path: "tests/RessourcesTest/FakeRessources/save.json",
      });

      // expect
      expect(tmp).to.be.eql(ml.listenerArray.map((item) => item.getProperty()));
    });
  });

  describe("test saving", function () {
    beforeEach("Clean old files if exist", function () {
      const fs = require("fs");
      if (fs.existsSync("tests/RessourcesTest/FakeRessources/fake_save.json"))
        fs.unlinkSync("tests/RessourcesTest/FakeRessources/fake_save.json");
    });

    afterEach("Clean old files if exist", function () {
      const fs = require("fs");
      if (fs.existsSync("tests/RessourcesTest/FakeRessources/fake_save.json"))
        fs.unlinkSync("tests/RessourcesTest/FakeRessources/fake_save.json");
    });

    it("should save the inserted objects", function (): void {
      // given
      const ml = new ManageListener();
      let expectedObj: ListenerRss[] = [];

      for (let i = 1; i < 10; i++) {
        const generatedInfo = {
          address: `fake.rss.service.${i}`,
          customfields: {
            "media:group": ["media:group"],
          },
          timeloop: i * 10,
        };
        expectedObj.push(new ListenerRss(generatedInfo));
        ml.addNewListener(generatedInfo);
      }

      // when
      ml.save("tests/RessourcesTest/FakeRessources/fake_save.json");
      const tmp = require("./RessourcesTest/FakeRessources/save.json");

      // expect
      expect(expectedObj.map((item) => item.getProperty())).to.be.eql(tmp);
    });
  });

  describe("test events", function () {
    beforeEach(function () {
      nock.disableNetConnect();
    });

    afterEach(function () {
      nock.cleanAll();
    });

    it.skip("should call correctly the events (no aggregation)", async function () {
      // given
      const clock = sinon.useFakeTimers();
      const ml = new ManageListener({
        path: "tests/RessourcesTest/RealRessources/save_no_history.json",
      });

      const spy = sinon.spy();

      const tabChannelId = [
        "UCOuIgj0CYCXCvjWywjDbauw",
        "UCh2YBKhYIy-_LtfCIn2Jycg",
        "UCY7klexd1qEqxgqYK6W7BVQ",
      ];

      ml.on("update", spy);
      tabChannelId.forEach((item: string) => {
        nock("https://www.youtube.com")
          .get(`/feeds/videos.xml?channel_id=${item}`)
          .replyWithFile(
            200,
            path.join(__dirname, `RessourcesTest/RealRessources/${item}.rss`),
            { "content-type": "text/xml", charset: "utf-8" }
          );
      });

      // when
      ml.startAll();

      await clock.tickAsync(2);
      await events.once(ml, "update");

      //expect
      expect(spy).to.have.been.calledThrice;

      // given
      spy.resetHistory();

      // when
      await clock.tickAsync(12000);

      ml.stopAll();

      //expect
      expect(spy).to.have.been.calledOnce;
    });

    it("should call correctly the events with a new entry (no aggregation)", function () {});
  });
});
